@@ -31,6 +31,7 @@ import org.apache.http.impl.client.*;
 import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
 import org.apache.http.protocol.HttpRequestExecutor;
 import org.apache.manifoldcf.core.interfaces.*;
+import org.apache.manifoldcf.connectorcommon.interfaces.*;
 import org.apache.manifoldcf.core.common.DateParser;
 import org.apache.manifoldcf.agents.interfaces.*;
 import org.apache.manifoldcf.agents.system.*;
@@ -45,10 +46,10 @@ import org.apache.http.auth.AuthScope;
 import org.apache.http.auth.Credentials;
 import org.apache.http.auth.UsernamePasswordCredentials;
 
-import org.apache.manifoldcf.core.util.URLEncoder;
 import org.apache.solr.client.solrj.SolrClient;
 import org.apache.solr.client.solrj.SolrServerException;
 import org.apache.solr.client.solrj.impl.CloudSolrClient;
+import org.apache.solr.client.solrj.impl.HttpSolrClient;
 import org.apache.solr.client.solrj.request.ContentStreamUpdateRequest;
 import org.apache.solr.client.solrj.request.UpdateRequest;
 import org.apache.solr.client.solrj.impl.XMLResponseParser;
@@ -66,6 +67,10 @@ import org.apache.solr.common.SolrInputDocument;
 
 import org.apache.commons.lang.StringUtils;
 
+import org.apache.solr.client.solrj.impl.HttpClientUtil;
+import org.apache.solr.client.solrj.impl.Krb5HttpClientBuilder;
+import org.apache.solr.client.solrj.impl.SolrHttpClientBuilder;
+
 /**
 * Posts an input stream to SOLR
 *
@@ -89,12 +94,12 @@ public class HttpPoster
   // Solrj connection-associated objects
   protected PoolingHttpClientConnectionManager connectionManager = null;
   protected SolrClient solrServer = null;
-  
+
   // Action URI pieces
   private final String postUpdateAction;
   private final String postRemoveAction;
   private final String postStatusAction;
-  
+
   // Attribute names
   private final String allowAttributeName;
   private final String denyAttributeName;
@@ -106,16 +111,17 @@ public class HttpPoster
   private final String fileNameAttributeName;
   private final String mimeTypeAttributeName;
   private final String contentAttributeName;
-  
+
   // Whether we use extract/update handler or not
   private final boolean useExtractUpdateHandler;
-  
-  // Use url encoding for field names
-  private final boolean useUrlEncoding;
-  
+
   // Document max length
   private final Long maxDocumentLength;
 
+  // Included and excluded mime types
+  private final Set<String> includedMimeTypes;
+  private final Set<String>excludedMimeTypes;
+
   // Commit-within flag
   private final String commitWithin;
 
@@ -124,7 +130,7 @@ public class HttpPoster
   private static final String NOTHING = "__NOTHING__";
   private static final String ID_METADATA = "lcf_metadata_id";
   private static final String COMMITWITHIN_METADATA = "commitWithin";
-  
+
   /** How long to wait before retrying a failed ingestion */
   private static final long interruptionRetryTime = 60000L;
 
@@ -138,16 +144,17 @@ public class HttpPoster
     String fileNameAttributeName, String mimeTypeAttributeName, String contentAttributeName,
     Long maxDocumentLength,
     String commitWithin, boolean useExtractUpdateHandler,
-    boolean useUrlEncoding)
+    final Set<String> includedMimeTypes, final Set<String> excludedMimeTypes,
+    boolean allowCompression)
     throws ManifoldCFException
   {
     // These are the paths to the handlers in Solr that deal with the actions we need to do
     this.postUpdateAction = updatePath;
     this.postRemoveAction = removePath;
     this.postStatusAction = statusPath;
-    
+
     this.commitWithin = commitWithin;
-    
+
     this.allowAttributeName = allowAttributeName;
     this.denyAttributeName = denyAttributeName;
     this.idAttributeName = idAttributeName;
@@ -159,12 +166,19 @@ public class HttpPoster
     this.mimeTypeAttributeName = mimeTypeAttributeName;
     this.contentAttributeName = contentAttributeName;
     this.useExtractUpdateHandler = useExtractUpdateHandler;
-    this.useUrlEncoding = useUrlEncoding;
+    this.includedMimeTypes = includedMimeTypes;
+    this.excludedMimeTypes = excludedMimeTypes;
+
     this.maxDocumentLength = maxDocumentLength;
-    
+
+    initializeKerberos();
+
     try
     {
-      CloudSolrClient cloudSolrServer = new CloudSolrClient(zookeeperHosts, new ModifiedLBHttpSolrClient(HttpClientUtil.createClient(null)));
+      CloudSolrClient cloudSolrServer = new CloudSolrClient.Builder()
+        .withZkHost(zookeeperHosts)
+        .withLBHttpSolrClient(new ModifiedLBHttpSolrClient(HttpClientUtil.createClient(null), allowCompression))
+        .build();
       cloudSolrServer.setZkClientTimeout(zkClientTimeout);
       cloudSolrServer.setZkConnectTimeout(zkConnectTimeout);
       cloudSolrServer.setDefaultCollection(collection);
@@ -188,16 +202,17 @@ public class HttpPoster
     String fileNameAttributeName, String mimeTypeAttributeName, String contentAttributeName,
     IKeystoreManager keystoreManager, Long maxDocumentLength,
     String commitWithin, boolean useExtractUpdateHandler,
-    boolean useUrlEncoding)
+    final Set<String> includedMimeTypes, final Set<String> excludedMimeTypes,
+    boolean allowCompression)
     throws ManifoldCFException
   {
     // These are the paths to the handlers in Solr that deal with the actions we need to do
     this.postUpdateAction = updatePath;
     this.postRemoveAction = removePath;
     this.postStatusAction = statusPath;
-    
+
     this.commitWithin = commitWithin;
-    
+
     this.allowAttributeName = allowAttributeName;
     this.denyAttributeName = denyAttributeName;
     this.idAttributeName = idAttributeName;
@@ -209,10 +224,13 @@ public class HttpPoster
     this.mimeTypeAttributeName = mimeTypeAttributeName;
     this.contentAttributeName = contentAttributeName;
     this.useExtractUpdateHandler = useExtractUpdateHandler;
-    this.useUrlEncoding = useUrlEncoding;
-    
+    this.includedMimeTypes = includedMimeTypes;
+    this.excludedMimeTypes = excludedMimeTypes;
+
     this.maxDocumentLength = maxDocumentLength;
 
+    initializeKerberos();
+
     String location = "";
     if (webapp != null)
       location = "/" + webapp;
@@ -224,7 +242,7 @@ public class HttpPoster
     }
 
     // Initialize standard solr-j.
-    
+
     SSLConnectionSocketFactory myFactory;
     if (keystoreManager != null)
     {
@@ -242,12 +260,12 @@ public class HttpPoster
       .register("https", myFactory)
       .build());
     connectionManager.setDefaultMaxPerRoute(1);
-    connectionManager.setValidateAfterInactivity(60000);
+    connectionManager.setValidateAfterInactivity(2000);
     connectionManager.setDefaultSocketConfig(SocketConfig.custom()
       .setTcpNoDelay(true)
       .setSoTimeout(socketTimeout)
       .build());
-    
+
     RequestConfig.Builder requestBuilder = RequestConfig.custom()
       .setCircularRedirectsAllowed(true)
       .setSocketTimeout(socketTimeout)
@@ -259,7 +277,7 @@ public class HttpPoster
       .setConnectionManager(connectionManager)
       .disableAutomaticRetries()
       .setDefaultRequestConfig(requestBuilder.build())
-      .setRedirectStrategy(new DefaultRedirectStrategy())
+      .setRedirectStrategy(new LaxRedirectStrategy())
       .setRequestExecutor(new HttpRequestExecutor(socketTimeout));
 
 
@@ -267,10 +285,17 @@ public class HttpPoster
     {
       CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
       Credentials credentials = new UsernamePasswordCredentials(userID, password);
-      if (realm != null)
-        credentialsProvider.setCredentials(new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, realm), credentials);
+      if (realm != null && realm.trim().length() > 0)
+      {
+        final AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, realm);
+        credentialsProvider.setCredentials(scope, credentials);
+        clientBuilder.addInterceptorFirst(new PreemptiveBasicAuthInterceptor(scope));
+      }
       else
+      {
         credentialsProvider.setCredentials(AuthScope.ANY, credentials);
+        clientBuilder.addInterceptorFirst(new PreemptiveBasicAuthInterceptor(AuthScope.ANY));
+      }
 
       clientBuilder.setDefaultCredentialsProvider(credentialsProvider);
     }
@@ -279,7 +304,22 @@ public class HttpPoster
 
 
     String httpSolrServerUrl = protocol + "://" + server + ":" + port + location;
-    solrServer = new ModifiedHttpSolrClient(httpSolrServerUrl, localClient, new XMLResponseParser());
+    solrServer = new ModifiedHttpSolrClient(httpSolrServerUrl, localClient, new XMLResponseParser(), allowCompression);
+  }
+
+  private static void initializeKerberos()
+  {
+
+    final String loginConfig = System.getProperty("java.security.auth.login.config");
+    if (loginConfig != null && loginConfig.trim().length() > 0) {
+      if (Logging.ingest.isInfoEnabled()) {
+        Logging.ingest.info("Using Kerberos for Solr Authentication");
+      }
+      Krb5HttpClientBuilder krbBuild = new Krb5HttpClientBuilder();
+      SolrHttpClientBuilder kb = krbBuild.getBuilder();
+      HttpClientUtil.setHttpClientBuilder(kb);
+    }
+
   }
 
   /** Shut down the poster.
@@ -287,13 +327,22 @@ public class HttpPoster
   public void shutdown()
   {
     if (solrServer != null)
-      solrServer.shutdown();
-    solrServer = null;
+    {
+      try
+      {
+        solrServer.close();
+      }
+      catch (IOException ioe)
+      {
+        // Eat this exception
+      }
+      solrServer = null;
+    }
     if (connectionManager != null)
       connectionManager.shutdown();
     connectionManager = null;
   }
-  
+
   /** Cause a commit to happen.
   */
   public void commitPost()
@@ -339,7 +388,7 @@ public class HttpPoster
       return;
     }
   }
-  
+
   /** Handle a RuntimeException.
   * Unfortunately, SolrCloud 4.6.x throws RuntimeExceptions whenever ZooKeeper is not happy.
   * We have to catch these too.  I've logged a ticket: SOLR-5678.
@@ -358,12 +407,14 @@ public class HttpPoster
         -1,
         true);
     }
-    throw e;
+    // Solr was not able to parse the request because it is malformed: skip the document
+    Logging.ingest.warn("Solr was unable to parse request during "+context+": "+e.getMessage(),e);
+    return;
   }
-  
+
   /** Handle a SolrServerException.
-  * These exceptions seem to be catch-all exceptions having to do either with misconfiguration or
-  * with underlying IO exceptions.
+  * These exceptions seem to be catch-all exceptions having to do with misconfiguration or
+  * underlying IO exceptions, or request parsing exceptions.
   * If this method doesn't throw an exception, it means that the exception should be interpreted
   * as meaning that the document or action is illegal and should not be repeated.
   */
@@ -375,8 +426,14 @@ public class HttpPoster
     {
       handleIOException((IOException)childException, context);
       return;
+    } else if (childException instanceof RuntimeException) {
+      handleRuntimeException((RuntimeException) childException, context);
+      return;
     }
-    throw new ManifoldCFException("Unhandled SolrServerException: "+e.getMessage());
+    //  Unknown exception, but a "Solr down" related issue does not end up here: log the error and skip this document or action
+    String message = "SolrServerException exception during "+context+": "+e.getMessage();
+    Logging.ingest.warn(message,e);
+    return;
   }
 
   /** Handle a SolrException.
@@ -407,22 +464,30 @@ public class HttpPoster
         throw new ManifoldCFException("Unexpected error: "+e2.getMessage());
       }
     }
-      
+
     // Use the exception text to determine the proper result.
     if (code == 500 && e.getMessage().indexOf("org.apache.tika.exception.TikaException") != -1)
       // Can't process the document, so don't keep trying.
       return;
 
+    // If code is 401, we should abort the job because security credentials are incorrect
+    if (code == 401)
+    {
+      String message = "Solr authorization failure, code "+code+": aborting job";
+      Logging.ingest.error(message);
+      throw new ManifoldCFException(message);
+    }
+
     // If the code is in the 400 range, the document will never be accepted, so indicate that.
     if (code >= 400 && code < 500)
       return;
-    
+
     // The only other kind of return code we know how to handle is 50x.
     // For these, we should retry for a while.
     if (code == 500)
     {
       long currentTime = System.currentTimeMillis();
-      
+
       // Log the error
       String message = "Solr exception during "+context+" ("+e.code()+"): "+e.getMessage();
       Logging.ingest.warn(message,e);
@@ -433,11 +498,11 @@ public class HttpPoster
         -1,
         true);
     }
-    
+
     // Unknown code: end the job.
     throw new ManifoldCFException("Unhandled Solr exception during "+context+" ("+e.code()+"): "+e.getMessage());
   }
-  
+
   /** Handle an IOException.
   * I'm not actually sure where these exceptions come from in SolrJ, but we handle them
   * as real I/O errors, meaning they should be retried.
@@ -449,7 +514,7 @@ public class HttpPoster
       throw new ManifoldCFException(e.getMessage(), ManifoldCFException.INTERRUPTED);
 
     long currentTime = System.currentTimeMillis();
-    
+
     if (e instanceof java.net.ConnectException)
     {
       // Server isn't up at all.  Try for a brief time then give up.
@@ -462,7 +527,7 @@ public class HttpPoster
         3,
         true);
     }
-    
+
     if (e instanceof java.net.SocketTimeoutException)
     {
       String message2 = "Socket timeout exception during "+context+": "+e.getMessage();
@@ -474,7 +539,7 @@ public class HttpPoster
         -1,
         false);
     }
-      
+
     if (e.getClass().getName().equals("java.net.SocketException"))
     {
       // In the past we would have treated this as a straight document rejection, and
@@ -501,8 +566,8 @@ public class HttpPoster
           3,
           false);
       }
-      
-      // Other socket exceptions are service interruptions - but if we keep getting them, it means 
+
+      // Other socket exceptions are service interruptions - but if we keep getting them, it means
       // that a socket timeout is probably set too low to accept this particular document.  So
       // we retry for a while, then skip the document.
       String message2 = "Socket exception during "+context+": "+e.getMessage();
@@ -515,31 +580,31 @@ public class HttpPoster
         false);
     }
 
-    // Otherwise, no idea what the trouble is, so presume that retries might fix it.
+    // Otherwise, no idea what the trouble is, so presume that retries might fix it, skip the document/action otherwise
     String message3 = "IO exception during "+context+": "+e.getMessage();
     Logging.ingest.warn(message3,e);
     throw new ServiceInterruption(message3,
       e,
       currentTime + interruptionRetryTime,
-      currentTime + 2L * 60L * 60000L,
-      -1,
-      true);
+      -1L,
+      3,
+      false);
   }
-  
+
   /**
   * Post the input stream to ingest
   *
    * @param documentURI is the document's uri.
    * @param document is the document structure to ingest.
    * @param arguments are the configuration arguments to pass in the post.  Key is argument name, value is a list of the argument values.
-   * @param keepAllMetadata
    * @param authorityNameString is the name of the governing authority for this document's acls, or null if none.
    * @param activities is the activities object, so we can report what's happening.   @return true if the ingestion was successful, or false if the ingestion is illegal.
-  * @throws ManifoldCFException, ServiceInterruption
+  * @throws ManifoldCFException
+  * @throws ServiceInterruption
   */
   public boolean indexPost(String documentURI,
-    RepositoryDocument document, Map<String,List<String>> arguments, Map<String, List<String>> sourceTargets,
-    boolean keepAllMetadata, String authorityNameString, IOutputAddActivity activities)
+    RepositoryDocument document, Map<String,List<String>> arguments,
+    String authorityNameString, IOutputAddActivity activities)
     throws ManifoldCFException, ServiceInterruption
   {
     if (Logging.ingest.isDebugEnabled())
@@ -551,7 +616,14 @@ public class HttpPoster
       return false;
     }
 
-    
+    // If not the right mime type, reject it.
+    // Note: this code added as part of CONNECTORS-1482 was incorrect!  Document filtering specified in the solr connector is always against the
+    // ORIGINAL mime type (which is what's in the document).  This why the checkMimeTypeIndexable second argument is always "true".
+    if ((includedMimeTypes !=null || excludedMimeTypes != null) && !checkMimeTypeIndexable(document.getMimeType(), true, includedMimeTypes, excludedMimeTypes)) {
+      activities.recordActivity(null,SolrConnector.INGEST_ACTIVITY,null,documentURI,activities.EXCLUDED_MIMETYPE,"Solr connector rejected document due to mime type restrictions: ("+document.getMimeType()+")");
+      return false;
+    }
+
     // Convert the incoming acls that we know about to qualified forms, and reject the document if
     // we don't know how to deal with its acls
     Map<String,String[]> aclsMap = new HashMap<String,String[]>();
@@ -563,7 +635,7 @@ public class HttpPoster
       String aclType = aclTypes.next();
       aclsMap.put(aclType,convertACL(document.getSecurityACL(aclType),authorityNameString,activities));
       denyAclsMap.put(aclType,convertACL(document.getSecurityDenyACL(aclType),authorityNameString,activities));
-      
+
       // Reject documents that have security we don't know how to deal with in the Solr plugin!!  Only safe thing to do.
       if (!aclType.equals(RepositoryDocument.SECURITY_TYPE_DOCUMENT) &&
         !aclType.equals(RepositoryDocument.SECURITY_TYPE_SHARE) &&
@@ -576,7 +648,7 @@ public class HttpPoster
 
     try
     {
-      IngestThread t = new IngestThread(documentURI,document,arguments,keepAllMetadata,sourceTargets,
+      IngestThread t = new IngestThread(documentURI,document,arguments,
                                         aclsMap,denyAclsMap);
       try
       {
@@ -704,7 +776,7 @@ public class HttpPoster
       {
         t.start();
         t.finishUp();
-        
+
         if (t.getActivityCode() != null)
           activities.recordActivity(t.getActivityStart(),SolrConnector.REMOVE_ACTIVITY,null,documentURI,t.getActivityCode(),t.getActivityDetails());
 
@@ -763,6 +835,41 @@ public class HttpPoster
 
   }
 
+  private final static Set<String> acceptableMimeTypes = new HashSet<String>();
+  static
+  {
+    acceptableMimeTypes.add("text/plain;charset=utf-8");
+    acceptableMimeTypes.add("text/plain;charset=ascii");
+    acceptableMimeTypes.add("text/plain;charset=us-ascii");
+    acceptableMimeTypes.add("text/plain; charset=utf-8");
+    acceptableMimeTypes.add("text/plain; charset=ascii");
+    acceptableMimeTypes.add("text/plain; charset=us-ascii");
+    acceptableMimeTypes.add("text/plain");
+  }
+
+  public static boolean checkMimeTypeIndexable(final String mimeType, final boolean useExtractUpdateHandler,
+    final Set<String> includedMimeTypes, final Set<String> excludedMimeTypes)
+  {
+    final String lowerMimeType = (mimeType==null)?null:mimeType.toLowerCase(Locale.ROOT);
+    if (useExtractUpdateHandler)
+    {
+      // Strip the charset off for this check
+      int index = lowerMimeType == null ? -1 : lowerMimeType.indexOf(";");
+      final String checkMimeType;
+      if (index != -1) {
+        checkMimeType = lowerMimeType.substring(0,index);
+      } else {
+        checkMimeType = lowerMimeType;
+      }
+      if (includedMimeTypes != null && !includedMimeTypes.contains(checkMimeType))
+        return false;
+      if (excludedMimeTypes != null && excludedMimeTypes.contains(checkMimeType))
+        return false;
+      return true;
+    }
+    return acceptableMimeTypes.contains(lowerMimeType);
+  }
+
   /** Convert an unqualified ACL to qualified form.
   * @param acl is the initial, unqualified ACL.
   * @param authorityNameString is the name of the governing authority for this document's acls, or null if none.
@@ -786,29 +893,14 @@ public class HttpPoster
     return new String[0];
   }
 
-  /** Conditionally encode */
-  protected String conditionallyEncode(String fieldName)
-  {
-    if (!useUrlEncoding)
-      return fieldName;
-    try
-    {
-      return java.net.URLEncoder.encode(fieldName, "UTF-8");
-    }
-    catch (IOException e)
-    {
-      throw new RuntimeException("Could not find utf-8 encoding!");
-    }
-  }
-  
   /** Write a field */
-  protected void writeField(ModifiableSolrParams out, String fieldName, String[] fieldValues)
+  protected static void writeField(ModifiableSolrParams out, String fieldName, String[] fieldValues)
   {
-    out.add(conditionallyEncode(fieldName), fieldValues);
+    out.add(fieldName, fieldValues);
   }
-  
+
   /** Write a field */
-  protected void writeField(ModifiableSolrParams out, String fieldName, List<String> fieldValues)
+  protected static void writeField(ModifiableSolrParams out, String fieldName, List<String> fieldValues)
   {
     String[] values = new String[fieldValues.size()];
     int i = 0;
@@ -817,11 +909,11 @@ public class HttpPoster
     }
     writeField(out, fieldName, values);
   }
-  
+
   /** Write a field */
-  protected void writeField(ModifiableSolrParams out, String fieldName, String fieldValue)
+  protected static void writeField(ModifiableSolrParams out, String fieldName, String fieldValue)
   {
-    out.add(conditionallyEncode(fieldName), fieldValue);
+    out.add(fieldName, fieldValue);
   }
 
   /** Output an acl level */
@@ -838,7 +930,7 @@ public class HttpPoster
       writeField(out,metadataDenyACLName,denyAcl[i]);
     }
   }
-  
+
   /**
     * Output an acl level in a SolrInputDocument
     */
@@ -862,11 +954,9 @@ public class HttpPoster
     protected final String documentURI;
     protected final RepositoryDocument document;
     protected final Map<String,List<String>> arguments;
-    protected final Map<String,List<String>> sourceTargets;
     protected final Map<String,String[]> aclsMap;
     protected final Map<String,String[]> denyAclsMap;
-    protected final boolean keepAllMetadata;
-    
+
     protected Long activityStart = null;
     protected Long activityBytes = null;
     protected String activityCode = null;
@@ -876,7 +966,7 @@ public class HttpPoster
     protected boolean rval = false;
 
     public IngestThread(String documentURI, RepositoryDocument document,
-      Map<String, List<String>> arguments, boolean keepAllMetadata, Map<String, List<String>> sourceTargets,
+      Map<String, List<String>> arguments,
       Map<String,String[]> aclsMap, Map<String,String[]> denyAclsMap)
     {
       super();
@@ -886,8 +976,6 @@ public class HttpPoster
       this.arguments = arguments;
       this.aclsMap = aclsMap;
       this.denyAclsMap = denyAclsMap;
-      this.sourceTargets = sourceTargets;
-      this.keepAllMetadata=keepAllMetadata;
     }
 
     public void run()
@@ -909,7 +997,7 @@ public class HttpPoster
           ContentStreamUpdateRequest contentStreamUpdateRequest = new ContentStreamUpdateRequest(postUpdateAction);
           if ( useExtractUpdateHandler )
           {
-            buildExtractUpdateHandlerRequest( length, is, contentType, contentName,
+            buildExtractUpdateHandlerRequest( length, is, contentType, (contentName==null || contentName.length()==0)?"docname":contentName,
               contentStreamUpdateRequest );
           }
           else
@@ -926,11 +1014,24 @@ public class HttpPoster
             UpdateResponse response;
             if ( useExtractUpdateHandler )
             {
-              response = contentStreamUpdateRequest.process( solrServer );
+              response = contentStreamUpdateRequest.process(solrServer);
             }
             else
             {
-              response = solrServer.add( currentSolrDoc );
+              final ModifiableSolrParams params = new ModifiableSolrParams();
+              // Write the arguments
+              for (final String name : arguments.keySet())
+              {
+                final List<String> values = arguments.get(name);
+                writeField(params, name, values);
+              }
+              final UpdateRequest req = new UpdateRequest();
+              req.setParams(params);
+              req.add(currentSolrDoc);
+              if (commitWithin != null) {
+                req.setCommitWithin(Integer.parseInt(commitWithin));
+              }
+              response =  req.process(solrServer);
             }
 
             // Successful completion
@@ -949,7 +1050,7 @@ public class HttpPoster
             activityBytes = new Long(length);
             activityDetails = e.getMessage() +
               ((e.getCause() != null)?": "+e.getCause().getMessage():"");
-            
+
             // Broken pipe exceptions we log specially because they usually mean
             // Solr has rejected the document, and the user will want to know that.
             if (e.getCause() != null && e.getCause().getClass().getName().equals("java.net.SocketException") &&
@@ -971,7 +1072,7 @@ public class HttpPoster
             activityCode = Integer.toString(e.code());
             activityDetails = e.getMessage() +
               ((e.getCause() != null)?": "+e.getCause().getMessage():"");
-            
+
             // Rethrow; we'll interpret at the next level
             throw e;
           }
@@ -980,7 +1081,7 @@ public class HttpPoster
         {
           if ((ioe instanceof InterruptedIOException) && (!(ioe instanceof java.net.SocketTimeoutException)))
             return;
-          
+
           activityStart = new Long(fullStartTime);
           activityCode = ioe.getClass().getSimpleName().toUpperCase(Locale.ROOT);
           activityDetails = ioe.getMessage();
@@ -1004,7 +1105,7 @@ public class HttpPoster
 
       // Write the id field
       outputDoc.addField( idAttributeName, documentURI );
-      
+
       if (contentAttributeName != null)
       {
         // Copy the content into a string.  This is a bad thing to do, but we have no choice given SolrJ architecture at this time.
@@ -1021,7 +1122,7 @@ public class HttpPoster
         }
         outputDoc.addField( contentAttributeName, sb.toString() );
       }
-      
+
       // Write the rest of the attributes
       if ( originalSizeAttributeName != null )
       {
@@ -1080,13 +1181,6 @@ public class HttpPoster
         writeACLsInSolrDoc(outputDoc,aclType,aclsMap.get(aclType),denyAclsMap.get(aclType));
       }
 
-      // Write the arguments
-      for ( String name : arguments.keySet() )
-      {
-        List<String> values = arguments.get( name );
-        outputDoc.addField( name, values );
-      }
-
       // Write the metadata, each in a field by itself
       buildSolrParamsFromMetadata( outputDoc );
 
@@ -1099,7 +1193,8 @@ public class HttpPoster
       throws IOException
     {
       ModifiableSolrParams out = new ModifiableSolrParams();
-          
+      Logging.ingest.debug("Solr: Writing document '"+documentURI);
+
       // Write the id field
       writeField(out,LITERAL+idAttributeName,documentURI);
       // Write the rest of the attributes
@@ -1108,7 +1203,7 @@ public class HttpPoster
         Long size = document.getOriginalSize();
         if (size != null)
           // Write value
-          writeField(out,LITERAL+modifiedDateAttributeName,size.toString());
+          writeField(out,LITERAL+originalSizeAttributeName,size.toString());
       }
       if (modifiedDateAttributeName != null)
       {
@@ -1143,7 +1238,7 @@ public class HttpPoster
         if (!StringUtils.isBlank(mimeType))
           writeField(out,LITERAL+mimeTypeAttributeName,mimeType);
       }
-          
+
       // Write the access token information
       // Both maps have the same keys.
       Iterator<String> typeIterator = aclsMap.keySet().iterator();
@@ -1162,22 +1257,24 @@ public class HttpPoster
 
       // Write the metadata, each in a field by itself
       buildSolrParamsFromMetadata(out);
-             
+
       // These are unnecessary now in the case of non-solrcloud setups, because we overrode the SolrJ posting method to use multipart.
       //writeField(out,LITERAL+"stream_size",String.valueOf(length));
       //writeField(out,LITERAL+"stream_name",document.getFileName());
-          
+
       // General hint for Tika
       if (!StringUtils.isBlank(document.getFileName()))
         writeField(out,"resource.name",document.getFileName());
-          
+
       // Write the commitWithin parameter
       if (commitWithin != null)
         writeField(out,COMMITWITHIN_METADATA,commitWithin);
 
       contentStreamUpdateRequest.setParams(out);
-          
+
       contentStreamUpdateRequest.addContentStream(new RepositoryDocumentStream(is,length,contentType,contentName));
+
+      Logging.ingest.debug("Solr: Done writing '"+documentURI+"'");
     }
 
     /**
@@ -1190,57 +1287,24 @@ public class HttpPoster
       */
     private void buildSolrParamsFromMetadata(ModifiableSolrParams out) throws IOException
     {
-      if (this.keepAllMetadata)
+      Iterator<String> iter = document.getFields();
+      while (iter.hasNext())
       {
-        Iterator<String> iter = document.getFields();
-        while (iter.hasNext())
-        {
-          String fieldName = makeSafeLuceneField(iter.next());
-          List<String> mappings = sourceTargets.get(fieldName);
-          if (mappings != null)
-            for (String newFieldName : mappings)
-              applySingleMapping(fieldName, out, newFieldName);
-          else // the fields not mentioned in the mapping are added only if we have set the keep all metadata=true.
-            applySingleMapping(fieldName, out, fieldName);
-        }
-      }
-      else
-      {
-        //don't keep all the metadata but only the ones in sourceTargets
-        for (String originalFieldName : sourceTargets.keySet())
-        {
-          List<String> mapping = sourceTargets.get(originalFieldName);
-          for (String newFieldName : mapping)
-            applySingleMapping(originalFieldName, out, newFieldName);
-        }
+        String originalFieldName = iter.next();
+        String fieldName = makeSafeLuceneField(originalFieldName);
+        Logging.ingest.debug("Solr: Saw field '"+originalFieldName+"'; converted to '"+fieldName+"'");
+        applySingleMapping(originalFieldName, out, fieldName);
       }
     }
 
     private void buildSolrParamsFromMetadata(SolrInputDocument outputDocument) throws IOException
     {
-      if (this.keepAllMetadata)
+      Iterator<String> iter = document.getFields();
+      while (iter.hasNext())
       {
-        Iterator<String> iter = document.getFields();
-        while (iter.hasNext())
-        {
-          String fieldName = makeSafeLuceneField(iter.next());
-          List<String> mappings = sourceTargets.get(fieldName);
-          if (mappings != null)
-            for (String newFieldName : mappings)
-              applySingleMapping(fieldName, outputDocument, newFieldName);
-          else // the fields not mentioned in the mapping are added only if we have set the keep all metadata=true.
-            applySingleMapping(fieldName, outputDocument, fieldName);
-        }
-      }
-      else
-      {
-        //don't keep all the metadata but only the ones in sourceTargets
-        for (String originalFieldName : sourceTargets.keySet())
-        {
-          List<String> mapping = sourceTargets.get(originalFieldName);
-          for (String newFieldName : mapping)
-            applySingleMapping(originalFieldName, outputDocument, newFieldName);
-        }
+        String originalFieldName = iter.next();
+        String fieldName = makeSafeLuceneField(originalFieldName);
+        applySingleMapping(originalFieldName, outputDocument, fieldName);
       }
     }
 
@@ -1348,7 +1412,7 @@ public class HttpPoster
         try
         {
           UpdateResponse response = new UpdateRequest(postRemoveAction).deleteById(documentURI).process(solrServer);
-            
+
           // Success
           activityStart = new Long(fullStartTime);
           activityCode = "OK";
@@ -1430,7 +1494,7 @@ public class HttpPoster
       return activityDetails;
     }
   }
-  
+
   /** Killable thread that does a commit.
   * Java 1.5 stopped permitting thread interruptions to abort socket waits.  As a result, it is impossible to get threads to shutdown cleanly that are doing
   * such waits.  So, the places where this happens are segregated in their own threads so that they can be just abandoned.
@@ -1573,7 +1637,7 @@ public class HttpPoster
     protected final long length;
     protected final String contentType;
     protected final String contentName;
-    
+
     public RepositoryDocumentStream(InputStream is, long length, String contentType, String contentName)
     {
       this.is = is;
@@ -1581,19 +1645,19 @@ public class HttpPoster
       this.contentType = contentType;
       this.contentName = contentName;
     }
-    
+
     @Override
     public Long getSize()
     {
       return new Long(length);
     }
-    
+
     @Override
     public InputStream getStream() throws IOException
     {
       return is;
     }
-    
+
     @Override
     public Reader getReader() throws IOException
     {
@@ -1619,7 +1683,7 @@ public class HttpPoster
   {
     /** Request parameters. */
     private ModifiableSolrParams params;
-    
+
     /**
      * Create a new SolrPing object.
      */
@@ -1627,7 +1691,7 @@ public class HttpPoster
       super(METHOD.GET, "/admin/ping");
       params = new ModifiableSolrParams();
     }
-    
+
     public SolrPing(String url)
     {
       super( METHOD.GET, url );
@@ -1648,45 +1712,45 @@ public class HttpPoster
     public ModifiableSolrParams getParams() {
       return params;
     }
-    
+
     /**
      * Remove the action parameter from this request. This will result in the same
      * behavior as {@code SolrPing#setActionPing()}. For Solr server version 4.0
      * and later.
-     * 
+     *
      * @return this
      */
     public SolrPing removeAction() {
       params.remove(CommonParams.ACTION);
       return this;
     }
-    
+
     /**
      * Set the action parameter on this request to enable. This will delete the
      * health-check file for the Solr core. For Solr server version 4.0 and later.
-     * 
+     *
      * @return this
      */
     public SolrPing setActionDisable() {
       params.set(CommonParams.ACTION, CommonParams.DISABLE);
       return this;
     }
-    
+
     /**
      * Set the action parameter on this request to enable. This will create the
      * health-check file for the Solr core. For Solr server version 4.0 and later.
-     * 
+     *
      * @return this
      */
     public SolrPing setActionEnable() {
       params.set(CommonParams.ACTION, CommonParams.ENABLE);
       return this;
     }
-    
+
     /**
      * Set the action parameter on this request to ping. This is the same as not
      * including the action at all. For Solr server version 4.0 and later.
-     * 
+     *
      * @return this
      */
     public SolrPing setActionPing() {
@@ -1699,10 +1763,8 @@ public class HttpPoster
   /** See CONNECTORS-956.  Make a safe lucene field name from a possibly
   * unsafe input field name from a repository connector.
   */
-  protected String makeSafeLuceneField(String inputField)
+  protected static String makeSafeLuceneField(String inputField)
   {
-    if (useUrlEncoding)
-      return inputField;
     StringBuilder sb = new StringBuilder();
     boolean isFirst = true;
     for (int i = 0; i < inputField.length(); i++)
@@ -1728,6 +1790,6 @@ public class HttpPoster
     }
     return sb.toString();
   }
-  
+
 }
 
